#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['dimutex']

package_data = \
{'': ['*'],
 'dimutex': ['.hypothesis/*',
             '.hypothesis/unicode_data/*',
             '.hypothesis/unicode_data/13.0.0/*',
             '.mypy_cache/*',
             '.mypy_cache/3.6/*',
             '.mypy_cache/3.6/_typeshed/*',
             '.mypy_cache/3.6/aiohttp/*',
             '.mypy_cache/3.6/async_timeout/*',
             '.mypy_cache/3.6/asyncio/*',
             '.mypy_cache/3.6/attr/*',
             '.mypy_cache/3.6/chardet/*',
             '.mypy_cache/3.6/collections/*',
             '.mypy_cache/3.6/concurrent/*',
             '.mypy_cache/3.6/concurrent/futures/*',
             '.mypy_cache/3.6/cryptography/*',
             '.mypy_cache/3.6/cryptography/hazmat/*',
             '.mypy_cache/3.6/cryptography/hazmat/backends/*',
             '.mypy_cache/3.6/cryptography/hazmat/backends/openssl/*',
             '.mypy_cache/3.6/cryptography/hazmat/bindings/*',
             '.mypy_cache/3.6/cryptography/hazmat/bindings/openssl/*',
             '.mypy_cache/3.6/cryptography/hazmat/primitives/*',
             '.mypy_cache/3.6/cryptography/hazmat/primitives/asymmetric/*',
             '.mypy_cache/3.6/cryptography/hazmat/primitives/ciphers/*',
             '.mypy_cache/3.6/cryptography/hazmat/primitives/kdf/*',
             '.mypy_cache/3.6/cryptography/hazmat/primitives/serialization/*',
             '.mypy_cache/3.6/cryptography/x509/*',
             '.mypy_cache/3.6/dimutex/*',
             '.mypy_cache/3.6/email/*',
             '.mypy_cache/3.6/gcloud/*',
             '.mypy_cache/3.6/gcloud/aio/*',
             '.mypy_cache/3.6/gcloud/aio/auth/*',
             '.mypy_cache/3.6/html/*',
             '.mypy_cache/3.6/http/*',
             '.mypy_cache/3.6/importlib/*',
             '.mypy_cache/3.6/json/*',
             '.mypy_cache/3.6/jwt/*',
             '.mypy_cache/3.6/logging/*',
             '.mypy_cache/3.6/multidict/*',
             '.mypy_cache/3.6/os/*',
             '.mypy_cache/3.6/pkg_resources/*',
             '.mypy_cache/3.6/requests/*',
             '.mypy_cache/3.6/requests/packages/*',
             '.mypy_cache/3.6/requests/packages/urllib3/*',
             '.mypy_cache/3.6/requests/packages/urllib3/packages/*',
             '.mypy_cache/3.6/requests/packages/urllib3/packages/ssl_match_hostname/*',
             '.mypy_cache/3.6/requests/packages/urllib3/util/*',
             '.mypy_cache/3.6/urllib/*',
             '.mypy_cache/3.6/yarl/*']}

install_requires = \
['aiohttp', 'gcloud-aio-auth']

extras_require = \
{'lint': ['bandit', 'flake8', 'isort', 'mypy', 'unify'],
 'test': ['pytest>=6.2.0', 'pytest-asyncio', 'pytest-cov']}

setup(name='dimutex',
      version='1.3.1',
      description='Mutex implementation for distributed systems.',
      author=None,
      author_email='Gram <gram@orsinium.dev>',
      url=None,
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.6',
     )
